__version__ = 1, 0, 8
__author__ = '竹永康'
__author_email__ = 'zhuyk4@lenovo.com'


class IsddcCommand:
    """
    Executes the incoming command directly at instantiation time.
    """
    code: int  # Command exit code
    raw_output: str  # Command output

    def __init__(
            self,
            cmd: str,
            *,
            timeout: int = None,
            ignore_timeout: bool = False
    ):
        """
        @param cmd:            Command string.
        @param timeout:        Command timeout, Default never times out.
        @param ignore_timeout: If true, no exception is thrown after naming timeout.
        """

    def raise_if_error(self):
        if self.code != 0:
            raise CommandError

    @property
    def status(self) -> bool:
        return self.code == 0

    @property
    def output(self) -> str:
        return process(self.raw_output)

    @property
    def code_output(self) -> 'Tuple[int, str]':
        return self.code, self.output

    @property
    def status_output(self) -> 'Tuple[bool, str]':
        return self.status, self.output

    @property
    def output_else_raise(self) -> str:
        if self.status:
            return self.output
        raise CommandError

    def output_else_define(self, define: 'Any' = None) -> 'Any':
        return self.output if self.status else define

    def is_contain_string(self, string: str) -> bool:
        return string in self.raw_output

    def output_if_contain_string_else_raise(self, string: str) -> str:
        if self.is_contain_string(string):
            return self.output
        raise CommandError

    def table_output_to_dict(self, split: str = None) -> list:
        return table2dict(self.output_else_raise, split=split)


def raise_if_error(cmd: str, *, timeout: int = None):
    IsddcCommand(cmd, timeout=timeout).raise_if_error()


def code(cmd: str, *, timeout: int = None) -> int:
    return IsddcCommand(cmd, timeout=timeout).code


def status(cmd: str, *, timeout: int = None) -> bool:
    return IsddcCommand(cmd, timeout=timeout).status


def output(cmd: str, *, timeout: int = None) -> str:
    return IsddcCommand(cmd, timeout=timeout).output


def raw_output(cmd: str, *, timeout: int = None) -> str:
    return IsddcCommand(cmd, timeout=timeout).raw_output


def code_output(cmd: str, *, timeout: int = None) -> 'Tuple[int, str]':
    return IsddcCommand(cmd, timeout=timeout).code_output


def status_output(cmd: str, *, timeout: int = None) -> 'Tuple[bool, str]':
    return IsddcCommand(cmd, timeout=timeout).status_output


def output_else_raise(cmd: str, *, timeout: int = None) -> str:
    return IsddcCommand(cmd, timeout=timeout).output_else_raise


def output_else_define(
        cmd: str,
        *,
        define: 'Any' = None,
        timeout: int = None
) -> 'Any':
    return IsddcCommand(
        cmd, timeout=timeout
    ).output_else_define(define)


def is_contain_string(
        cmd: str,
        *,
        string: str = None,
        timeout: int = None
) -> bool:
    return IsddcCommand(
        cmd, timeout=timeout
    ).is_contain_string(string)


def output_if_contain_string_else_raise(
        cmd: str,
        *,
        string: str = None,
        timeout: int = None
) -> str:
    return IsddcCommand(
        cmd, timeout=timeout
    ).output_if_contain_string_else_raise(string)


def table_output_to_dict(cmd: str, *, timeout: int = None) -> list:
    return IsddcCommand(cmd, timeout=timeout).table_output_to_dict()


def table2dict(table: str, *, split: str = None) -> list:
    """Used to turn the command output result with a title
    into a dictionary, such as `kubectl get pod -n isddc`."""


class CommandError(Exception):
    pass


class _______i________s________d_______d_______c_______:
    import sys

    __import__(f'{__name__}.i {__name__[6:]}')
    ipack = sys.modules[__name__]
    icode = getattr(ipack, f'i {__name__[6:]}')

    for iname, ifunc in globals().items():
        if iname[0] != '_' and hasattr(icode, iname):
            setattr(ipack, iname, getattr(icode, iname))


icmd = IsddcCommand

from typing import Any, Tuple
